@@ -1,5 +1,5 @@
 """
-perlito.python.runtime
+Perlito__Python__Runtime
 
 DESCRIPTION
 
@@ -11,7 +11,7 @@ The Pugs Team E<lt>perl6-compiler@perl.orgE<gt>.
 
 COPYRIGHT
 
-Copyright 2010 by Flavio Soibelmann Glock and others.
+Copyright 2010, 2011 by Flavio Soibelmann Glock and others.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -26,8 +26,10 @@ import __builtin__
 
 __all__ = ['mp6_print', 'mp6_say', 'mp6_warn', 
            'mp6_to_num', 'mp6_to_scalar', 'mp6_to_bool', 'mp6_isa',
-           'mp6_join', 'mp6_index', 'mp6_perl',
-           'mp6_Undef', 'mp6_Array', 'mp6_Hash',
+           'mp6_and', 'mp6_or', 'mp6_defined_or',
+           'mp6_join', 'mp6_index', 'mp6_perl', 'mp6_id',
+           'mp6_Mu', 
+           'mp6_Array', 'mp6_Hash', 'mp6_Scalar',
            'mp6_Return',
            'Perlito__Match',
            'Perlito__Grammar', 'Perlito__Grammar_proto', 
@@ -55,21 +57,33 @@ def mp6_to_scalar(v):
         return v
 
 def mp6_to_bool(o):
-    if type(o) == type(False):
-        return o
-    if type(o) == type(1):
-        return o != 0
-    if type(o) == type(1.1):
-        return o != 0
-    if type(o) == type("aa"):
-        return o != "" and o != "0"
-    if type(o) == type([]):
-        return len(o) != 0
-    if type(o) == type({}):
-        return len(o.keys) != 0
-    if type(o) == type(None):
+    try: 
+        return o.f_bool()
+    except AttributeError:
+        if type(o) == type(False):
+            return o
+        if type(o) == type(1):
+            return o != 0
+        if type(o) == type(1.1):
+            return o != 0
+        if type(o) == type("aa"):
+            return o != "" and o != "0"
         return False
-    return o.__nonzero__()
+
+def mp6_and(x, y):
+    if mp6_to_bool(x):
+        return y()
+    return x
+
+def mp6_or(x, y):
+    if mp6_to_bool(x):
+        return x
+    return y()
+
+def mp6_defined_or(x, y):
+    if mp6_isa(x, 'Mu'):
+        return y()
+    return x
 
 def mp6_isa(v, name):
     try:
@@ -81,7 +95,7 @@ def mp6_isa(v, name):
             return True
         if name == 'Str' and type(v) == type("aa"):
             return True
-        print "Warning: Can't calculate .isa() on", v
+        mp6_warn("Warning: Can't calculate .isa() on", mp6_perl(v))
         return False
 
 def mp6_join(l, s):
@@ -122,12 +136,14 @@ class mp6_Array:
         return str(self.l)
     def __int__(self):
         return len(self.l)
-    def __nonzero__(self):
+    def f_bool(self):
         return len(self.l) > 0
     def __iter__(self):
         return self.l.__iter__()
     def f_perl(self):
         return mp6_perl(self.l)
+    def f_elems(self):
+        return len(self.l)
     def f_extend(self, l):
         try:
             self.l.extend(l.l)
@@ -139,24 +155,37 @@ class mp6_Array:
         try:
             return self.l.pop()
         except IndexError:
-            return mp6_Undef()
-    def f_set(self, i, s):
+            return mp6_Mu()
+    def f_set(self, v):
+        self.l = []
+        v = v.l
+        for x in range(0, len(v)):
+            try:
+                self.l.append(v[x].f_get())
+            except AttributeError:
+                self.l.append(v[x])
+        return self
+    def f_index_set(self, i, s):
+        try:
+            s = s.f_get()
+        except AttributeError:
+            1
         while True:
             try:
                 self.l[i] = s
                 return s
             except IndexError:
-                self.l.append( mp6_Undef() )
+                self.l.append( mp6_Mu() )
     def f_shift(self):
         try:
             return self.l.pop(0)
         except IndexError:
-            return mp6_Undef()
+            return mp6_Mu()
     def f_index(self, i):
         try:
-            return self.l[i]
+            return mp6_Mu_get_proxy(self, i, self.l[i])
         except IndexError:
-            return mp6_Undef()
+            return mp6_Mu_get_proxy(self, i, mp6_Mu())
     def f_isa(self, name):
         return name == 'Array'
     def str(self):
@@ -170,7 +199,7 @@ class mp6_Hash:
         return str(self.l)
     def __int__(self):
         return len(self.l)
-    def __nonzero__(self):
+    def f_bool(self):
         return len(self.l) > 0
     def __iter__(self):
         return self.l.__iter__()
@@ -182,22 +211,41 @@ class mp6_Hash:
         return self.l.values()
     def keys(self):
         return self.l.keys()
+    def f_pairs(self):
+        out = [];
+        for i in self.keys():
+            out.append( Pair(v_key=i, v_value=self.__getitem__(i)) )
+        return out
     def has_key(self, k):
         return self.l.has_key(k)
-    def f_set(self, i, s):
-        self.l[i] = s
+    def f_set(self, v):
+        self.l = {}
+        v = v.l
+        for x in v.keys():
+            try:
+                self.l[x] = v[x].f_get()
+            except AttributeError:
+                self.l[x] = v[x]
+        return self
+    def f_index_set(self, i, s):
+        try:
+            self.l[i] = s.f_get()
+        except AttributeError:
+            self.l[i] = s
         return s
+    def f_update(self, h):
+        self.l.update(h)
+        return self
     def f_lookup(self, i):
         try:
-            return self.l[i]
+            return mp6_Mu_get_proxy(self, i, self.l[i])
         except KeyError:
-            return mp6_Undef()
+            return mp6_Mu_get_proxy(self, i, mp6_Mu())
     def f_isa(self, name):
         return name == 'Hash'
     def str(self):
         return str(self.l)
 
-
 class mp6_Mu:
     def __str__(self):
         return ""
@@ -205,7 +253,7 @@ class mp6_Mu:
         return 0
     def __float__(self):
         return 0.0
-    def __nonzero__(self):
+    def f_bool(self):
         return False
     def __add__(self, x):
         return x
@@ -218,6 +266,95 @@ class mp6_Mu:
     def f_isa(self, name):
         return name == 'Mu'
 
+class mp6_Scalar:
+    def __init__(self, v=mp6_Mu()):
+        self.v = v
+    def __getattr__(self,name):
+        return getattr(self.v, name)
+    def __str__(self):
+        return str(self.v)
+    def f_perl(self):
+        return mp6_perl(self.v)
+    def f_id(self):
+        return mp6_id(self.v)
+    def f_bool(self):
+        return mp6_to_bool(self.v)
+    def f_set(self, v):
+        try:
+            self.v = v.f_get()
+        except AttributeError:
+            self.v = v
+    def f_get(self):
+        return self.v
+    def f_lookup(self, i):
+        try:
+            return self.v.f_lookup(i)
+        except AttributeError:
+            if self.v.f_isa('Mu'):
+                return mp6_Mu_lookup_proxy(self, i)
+            return self.v.f_lookup(i)
+    def f_index(self, i):
+        try:
+            return self.v.f_index(i)
+        except AttributeError:
+            if self.v.f_isa('Mu'):
+                return mp6_Mu_index_proxy(self, i)
+            return self.v.f_index(i)
+
+class mp6_Mu_get_proxy(mp6_Mu):
+    def __init__(self, parent, i, v):
+        self.parent = parent
+        self.i = i
+        self.v = v
+    def __getattr__(self,name):
+        return getattr(self.v, name)
+    def __str__(self):
+        return str(self.v)
+    def f_id(self):
+        return mp6_id(self.v)
+    def f_bool(self):
+        return mp6_to_bool(self.v)
+    def f_get(self):
+        return self.v
+    def f_set(self, v):
+        self.v = v
+        self.parent.f_index_set(self.i, v)
+    def f_lookup(self, i):
+        try:
+            return self.v.f_lookup(i)
+        except AttributeError:
+            return mp6_Mu_lookup_proxy(self, i)
+    def f_index(self, i):
+        try:
+            return self.v.f_index(i)
+        except AttributeError:
+            return mp6_Mu_index_proxy(self, i)
+
+class mp6_Mu_index_proxy(mp6_Mu):
+    def __init__(self, parent, i):
+        self.parent = parent
+        self.i = i
+    def f_set(self, v):
+        Array_a = mp6_Array([])
+        Array_a.f_index_set(self.i, v)
+        self.parent.f_set(Array_a)
+    def f_lookup(self, i):
+        return mp6_Mu_lookup_proxy(self, i)
+    def f_index(self, i):
+        return mp6_Mu_index_proxy(self, i)
+
+class mp6_Mu_lookup_proxy(mp6_Mu):
+    def __init__(self, parent, i):
+        self.parent = parent
+        self.i = i
+    def f_set(self, v):
+        Hash_a = mp6_Hash({})
+        Hash_a.f_index_set(self.i, v)
+        self.parent.f_set(Hash_a)
+    def f_lookup(self, i):
+        return mp6_Mu_lookup_proxy(self, i)
+    def f_index(self, i):
+        return mp6_Mu_index_proxy(self, i)
 
 class mp6_Return(Exception):
     def __init__(self, value):
@@ -227,40 +364,40 @@ class mp6_Return(Exception):
 
 class Perlito__Match:
     def __init__(self, **arg):
-        self.v_m = {}
+        self.v_m = mp6_Hash({})
         self.v_to = 0
+        self.v_capture = mp6_Scalar()
         self.__dict__.update(arg)
     def __setattr__(v_self, k, v):
         v_self.__dict__[k] = v
         return v
     def __str__(self):
         if mp6_to_bool(self.v_bool):
-            try:
+            if not(mp6_isa(self.v_capture,'Mu')): 
                 return str(self.v_capture)
-            except AttributeError:
-                return self.v_str[self.v_from:self.v_to]
+            return self.v_str[self.v_from:self.v_to]
         return ''
-    def __nonzero__(self):
+    def f_bool(self):
         return mp6_to_bool(self.v_bool)
-    def __setitem__(self, k, v):
-        self.v_m[k] = v
-    def f_set(self, k, v):
-        self.v_m[k] = v
-        return v
+    def f_set(self, m):
+        self.v_m  = m.v_m
+        self.v_to = m.v_to
+        self.v_str = m.v_str
+        self.v_from = m.v_from
+        self.v_bool = m.v_bool
+        self.v_capture = m.v_capture
+    def f_capture(self):
+        return self.v_capture
+    def f_index_set(self, k, v):
+        return self.v_m.f_index_set(k, v)
     def f_lookup(self, k):
-        try:
-            return self.v_m[k]
-        except KeyError:
-            return mp6_Undef()
-    def __getitem__(self, k):
-        return self.v_m[k] 
+        return self.v_m.f_lookup(k)
     def f_scalar(self):
         if mp6_to_bool(self.v_bool):
-            try:
+            if not(mp6_isa(self.v_capture,'Mu')): 
                 return self.v_capture
-            except AttributeError:
-                return self.v_str[self.v_from:self.v_to]
-        return mp6_Undef()
+            return self.v_str[self.v_from:self.v_to]
+        return self.v_capture
     def has_key(self, k):
         return self.v_m.has_key(k)
     def f_from(self):
@@ -269,6 +406,9 @@ class Perlito__Match:
         return self.v_to
     def f_isa(self, name):
         return name == 'Match'
+Perlito__Match_proto = Perlito__Match()
+__builtin__.Perlito__Match = Perlito__Match
+__builtin__.Perlito__Match_proto = Perlito__Match_proto
 
 
 try:        
@@ -305,6 +445,8 @@ except NameError:
         def f_isa(self, name):
             return name == 'Grammar'
 Perlito__Grammar_proto = Perlito__Grammar()
+__builtin__.Perlito__Grammar = Perlito__Grammar
+__builtin__.Perlito__Grammar_proto = Perlito__Grammar_proto
 
 
 class Main:
@@ -312,12 +454,10 @@ class Main:
         self.__dict__.update(arg)
     def __setattr__(v_self, k, v):
         v_self.__dict__[k] = v
-    def __nonzero__(self):
+    def f_bool(self):
         return 1
     def f_isa(v_self, name):
         return name == 'Main'
-    def f_newline(self):
-        return "\n"
     def f_lisp_escape_string(self, s):
         o = s.replace( "\\", "\\\\");
         o = o.replace( '"', "\\\"");
@@ -350,7 +490,7 @@ class IO:
         v_self.__dict__.update(arg)
     def __setattr__(v_self, k, v):
         v_self.__dict__[k] = v
-    def __nonzero__(self):
+    def f_bool(self):
         return 1
     def f_isa(v_self, name):
         return name == 'IO'
@@ -361,14 +501,26 @@ __builtin__.IO = IO
 __builtin__.IO_proto = IO_proto
 
 
-def _dump(o):
+def _dump(o, seen):
+    try:
+        if seen[id(o)]:
+            return '*recursive*'
+    except KeyError:
+        None
+    seen[id(o)] = True
     out = [];
     for i in o.__dict__.keys():
         out.append(i[2:] + " => " + mp6_perl(o.__dict__[i]))
     name = o.__class__.__name__.replace( "__", "::");
     return name + ".new(" + ", ".join(out) + ")";
 
-def mp6_perl(o):
+def mp6_id(o):
+    try:
+        return o.f_id()
+    except AttributeError:
+        return id(o)
+
+def mp6_perl(o, seen={}):
     try:
         return o.f_perl()
     except AttributeError:
@@ -381,14 +533,26 @@ def mp6_perl(o):
         if type(o) == type("aa"):
             return '"' + Main_proto.f_javascript_escape_string(o) + '"'
         if type(o) == type([]):
+            try:
+                if seen[id(o)]:
+                    return '*recursive*'
+            except KeyError:
+                None
+            seen[id(o)] = True
             return "[" + ", ".join(map(lambda x: mp6_perl(x), o)) + "]"
         if type(o) == type({}):
+            try:
+                if seen[id(o)]:
+                    return '*recursive*'
+            except KeyError:
+                None
+            seen[id(o)] = True
             out = [];
             for i in o.keys():
                 out.append(i + " => " + mp6_perl(o[i]))
             return "{" + ", ".join(out) + "}";
-        return _dump(o)
+        return _dump(o, seen)
 
-__builtin__.List_ARGS = [mp6_Array([])]
-List_ARGS[0].l.extend(sys.argv[1:])
+__builtin__.List_ARGS = mp6_Array([])
+List_ARGS.l.extend(sys.argv[1:])
 
